/* 
 * This file is part of fuelpad, the fuel diary
 *
 * Copyright (c) 2007, 2008 Julius Luukko <julle.luukko@quicknet.inet.fi>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ui.h"
#include "uireport.h"

/* Report window */
#define REPORTYEARS 4
#define HEADERLINE 0.1
#define LEFTMARGIN 0.15
#define INDENT 0.05
#define FIRSTCOLUMN 0.4
#define COLUMNSEP 0.1
#define HEADING1SIZE 16
#define HEADING2SIZE 14
#define NORMALSIZE 12

#define CONSUMETAG "CONSUMETAG"
#define COSTPERKMTAG "COSTPERKMTAG"
#define COSTTAG "COSTTAG"
#define PRICEPERUNITTAG "PRICEPERUNITTAG"

extern unit_t curunit;
extern gboolean carchanged;

static gboolean delete_reportwin( GtkWidget *widget,
				  GdkEvent  *event,
				  gpointer   data )
{
  extern AppUIData ui;

  gtk_widget_destroy(GTK_WIDGET(ui.app->reportwindow));
  return FALSE;
}

void destroy_reportwin( GtkWidget *widget,
		      gpointer   data )
{
  AppUIData *a;

  a=(AppUIData *)data;

  gtk_widget_destroy(GTK_WIDGET(a->app->reportwindow));
  a->app->reportwindow=NULL;
}

void report_window_fullscreen(gpointer data)
{
  AppUIData *a;

  a=(AppUIData *)data;
  a->reportfullscreen = !a->reportfullscreen;
  if (a->reportfullscreen) {
    gtk_window_fullscreen(GTK_WINDOW(a->app->reportwindow));
  } else {
    gtk_window_unfullscreen(GTK_WINDOW(a->app->reportwindow));
  }
}

void cb_report_fullscreen( GtkAction * action, gpointer data )
{
  report_window_fullscreen(data);
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(((AppUIData *)(data))->report_menu_item_fullscreen),
				 ((AppUIData *)(data))->reportfullscreen);
}

#if PLAINGTK == 0
void report_scroll(AppUIData *pui, gboolean dir)
{
  GtkAdjustment *vertical;

  vertical = gtk_scrolled_window_get_vadjustment((GtkScrolledWindow *)pui->reportscrollwin);
  if (dir) {
    vertical->value -= vertical->step_increment;
    if (vertical->value<vertical->lower)
      vertical->value = vertical->lower;
  }
  else {
    vertical->value += vertical->step_increment;
    if (vertical->value>vertical->upper)
      vertical->value = vertical->upper;
  }
  gtk_scrolled_window_set_vadjustment((GtkScrolledWindow *)pui->reportscrollwin,vertical);
  gtk_adjustment_value_changed(vertical);
}

gboolean report_key_press_cb(GtkWidget * widget, GdkEventKey * event,
			     AppUIData *pui)
{
  switch (event->keyval) {

    case GDK_F6:
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->report_menu_item_fullscreen),
				     !pui->reportfullscreen);
      return TRUE;
    case GDK_Down:
      report_scroll(pui,FALSE);
      return TRUE;
      break;
    case GDK_Up:
      report_scroll(pui,TRUE);
      return TRUE;
    case GDK_F7:
      report_zoom(pui->reportcanvas, ZOOMIN);
      return TRUE;
    case GDK_F8:
      report_zoom(pui->reportcanvas, ZOOMOUT);
      return TRUE;
      break;
    }

    return FALSE;
}
#endif

#if PLAINGTK == 1
void create_reportwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
#else
void create_reportwin_menu(HildonWindow *window, AppUIData *pui)
#endif
{
  GtkWidget *main_menu;
  GtkWidget *menu_view;
  GtkWidget *item_view;
  GtkWidget *menu_zoom;
  GtkWidget *item_save;
  GtkWidget *item_zoom;
  GtkWidget *item_zoom_in;
  GtkWidget *item_zoom_out;
  GtkWidget *item_exit;
#if PLAINGTK == 1
  GtkWidget *menubar;
  GtkWidget *item_file;
#endif
 
  main_menu = gtk_menu_new();
  menu_view = gtk_menu_new();
  menu_zoom = gtk_menu_new();

  /* Create main menu items */
  item_save = gtk_menu_item_new_with_label(_("Save report..."));
  item_view = gtk_menu_item_new_with_label(_("View"));
  item_zoom = gtk_menu_item_new_with_label(_("Zoom"));
  item_exit = gtk_menu_item_new_with_label(_("Close window"));

  /* Create view menu items */
  pui->report_menu_item_fullscreen = gtk_check_menu_item_new_with_label(_("Full screen"));
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->report_menu_item_fullscreen),
				 pui->reportfullscreen);

  /* Create zoom menu items */
  item_zoom_in = gtk_menu_item_new_with_label(_("Zoom in"));
  item_zoom_out = gtk_menu_item_new_with_label(_("Zoom out"));

  /* Add menu items to right menus */
  /* Main menu */
  gtk_menu_append(main_menu, item_save);
  gtk_menu_append(main_menu, gtk_separator_menu_item_new());
  gtk_menu_append(main_menu, item_view);
  gtk_menu_append(main_menu, gtk_separator_menu_item_new());
  gtk_menu_append(main_menu, item_zoom);
  gtk_menu_append(main_menu, gtk_separator_menu_item_new());
  gtk_menu_append(main_menu, item_exit);

  /* View menu */
  gtk_menu_append(menu_view, pui->report_menu_item_fullscreen);

  /* Zoom menu */
  gtk_menu_append(menu_zoom, item_zoom_in);
  gtk_menu_append(menu_zoom, item_zoom_out);

#if PLAINGTK == 1
  menubar = gtk_menu_bar_new();
  gtk_box_pack_start (GTK_BOX (root), menubar, FALSE, FALSE, 2);
  gtk_widget_show (menubar);
  item_file = gtk_menu_item_new_with_label ("fuelpad");
  gtk_widget_show (item_file);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (item_file), main_menu);
  gtk_menu_bar_append (GTK_MENU_BAR (menubar), item_file);
#else
  hildon_window_set_menu(HILDON_WINDOW(window), GTK_MENU(main_menu));
#endif
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_view), menu_view);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_zoom), menu_zoom);

  /* Attach the callback functions to the activate signal */
  g_signal_connect(G_OBJECT(item_save), "activate",
		   G_CALLBACK(callback_save_report), pui->reportcanvas);
  g_signal_connect(G_OBJECT(item_zoom_in), "activate",
		   G_CALLBACK(callback_report_zoom_in), pui->reportcanvas);
  g_signal_connect(G_OBJECT(item_zoom_out), "activate",
		   G_CALLBACK(callback_report_zoom_out), pui->reportcanvas);
  g_signal_connect_swapped(G_OBJECT(item_exit), "activate",
			   G_CALLBACK(delete_reportwin), G_OBJECT(window));

  g_signal_connect(G_OBJECT(pui->report_menu_item_fullscreen), "toggled",
		   G_CALLBACK(cb_report_fullscreen), pui);

  /* Make all menu widgets visible */
  gtk_widget_show_all(GTK_WIDGET(main_menu));
}

gdouble report_init_line(gdouble y)
{
  return y;
}

gdouble report_line_down(gdouble y, gint height, 
			 gdouble numlines, gint page_height)
{
  return y+numlines*height/YSCALE(page_height);
}

GtkPlotCanvasChild *report_put_text(GtkWidget *canvas,
				    gdouble x1, gdouble y1,
				    const gchar *font, gint height,
				    GtkJustification justification,
				    const gchar *real_text)
{
  GtkPlotCanvasChild *child;

  child = gtk_plot_canvas_text_new(font, height, 0, NULL, NULL, TRUE,
				   justification,
				   real_text);
  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, x1, YSCALE(y1), 0, 0);

  return child;
}

void report_convert_to_iso8859_15(GtkWidget *canvas)
{
  gchar *str;
  GList* pchild;
  GtkPlotCanvasText *apu;

  pchild=g_list_first(GTK_PLOT_CANVAS(canvas)->childs);

  for (;pchild != NULL;pchild=g_list_next(pchild)) {
    if (GTK_IS_PLOT_CANVAS_TEXT(pchild->data)) {
      str=g_convert(GTK_PLOT_CANVAS_TEXT(pchild->data)->text.text, -1, 
		    "ISO-8859-15", "UTF-8", NULL, NULL, NULL);
      apu=GTK_PLOT_CANVAS_TEXT(pchild->data);
      gtk_plot_canvas_text_set_attributes(apu, NULL, apu->text.height, 
					  apu->text.angle, 
					  NULL, NULL, 
					  apu->text.transparent,
					  apu->text.justification,
					  str);
      g_free(str);
    }
  }
}

void callback_save_report( GtkAction * action, gpointer data )
{
  gint pagesize;
  gchar *filename = NULL;
  filename = interface_file_chooser ("Save postscript file",GTK_FILE_CHOOSER_ACTION_SAVE);

  if (filename == NULL) {
    filename = "NULL";
  }
  else {
    report_convert_to_iso8859_15(GTK_WIDGET(data));
    if (GTK_PLOT_CANVAS(data)->width == GTK_PLOT_A4_W)
      pagesize=GTK_PLOT_A4;
    else
      pagesize=GTK_PLOT_LETTER;
    gtk_plot_canvas_export_ps(GTK_PLOT_CANVAS(data), filename, GTK_PLOT_PORTRAIT, FALSE, pagesize);
    ui_show_banner(_("Report saved."));
  }
}

gdouble get_magnification(GtkWidget *canvas)
{
  return GTK_PLOT_CANVAS(canvas)->magnification;
}

void report_zoom(GtkWidget *canvas, zoom_t dir)
{
  static gdouble zoom=1.0;
  gdouble curmag;

  curmag = get_magnification(GTK_WIDGET(canvas));

  if (dir==ZOOMIN) {
    if (zoom<1.0) {
      zoom += 0.1;
      gtk_plot_canvas_set_magnification(GTK_PLOT_CANVAS(canvas), 1.1*curmag);
    }
    else
      ui_show_banner(_("Can't zoom in more"));
  }
  else {
    if (zoom>0.1) {
      zoom -= 0.1;
      gtk_plot_canvas_set_magnification(GTK_PLOT_CANVAS(canvas), curmag/1.1);
    }
    else
      ui_show_banner(_("Can't zoom out more"));
  }
  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(canvas));
}

void callback_report_zoom_in( GtkAction * action, gpointer data )
{
  report_zoom(GTK_WIDGET(data), ZOOMIN);
}

void callback_report_zoom_out( GtkAction * action, gpointer data )
{
  report_zoom(GTK_WIDGET(data), ZOOMOUT);
}

void fill_year_combo(GtkWidget *combo)
{
  char listitemtext[ENTRYDATEMAX]="";
  gint i;

  i=0;

  if (SQLITE_OK == sqlite3_bind_int(ppStmtGetYears, 1, currentcar)) {
    while(SQLITE_ROW == sqlite3_step(ppStmtGetYears)) {
      g_snprintf(listitemtext, ENTRYDATEMAX, "%s", 
		 sqlite3_column_text(ppStmtGetYears,0));
      gtk_combo_box_append_text(GTK_COMBO_BOX (combo),listitemtext);
      i++;
    }
    sqlite3_reset(ppStmtGetYears);
    gtk_combo_box_set_active(GTK_COMBO_BOX (combo),i-1);
  }
}

GtkWidget *create_year_combo(void)
{
  GtkWidget *combo;

  combo = gtk_combo_box_new_text();
  fill_year_combo(combo);
  return combo;
}

void update_yearcombo_cb(GtkWidget *widget, gpointer combo)
{
  GtkTreeModel *model;
  gint i, count;

  model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
  count = gtk_tree_model_iter_n_children(model, NULL);
  for (i=0; i < count; i++)
    gtk_combo_box_remove_text(GTK_COMBO_BOX(combo), 0);
  fill_year_combo(GTK_WIDGET(combo));
}

#if PLAINGTK == 1
static void create_report_toolbar(GtkWidget *window, GtkWidget *vbox, GtkWidget *canvas)
#else
static void create_report_toolbar(HildonWindow *window, GtkWidget *canvas)
#endif
{
  GtkWidget *toolbar;
  GtkToolItem *tb_save;
  GtkToolItem *tb_zoom_in;
  GtkToolItem *tb_zoom_out;
  GtkToolItem *tb_close;

  /* Create toolbar */
  toolbar = gtk_toolbar_new();

  /* Create toolbar button items */
  tb_save = gtk_tool_button_new_from_stock(GTK_STOCK_SAVE);
  tb_zoom_in = gtk_tool_button_new_from_stock(GTK_STOCK_ZOOM_IN);
  tb_zoom_out = gtk_tool_button_new_from_stock(GTK_STOCK_ZOOM_OUT);
  tb_close = gtk_tool_button_new_from_stock(GTK_STOCK_QUIT);

  gtk_tool_item_set_expand(GTK_TOOL_ITEM(tb_save),TRUE);
  gtk_tool_item_set_expand(GTK_TOOL_ITEM(tb_zoom_in),TRUE);
  gtk_tool_item_set_expand(GTK_TOOL_ITEM(tb_zoom_out),TRUE);
  gtk_tool_item_set_expand(GTK_TOOL_ITEM(tb_close),TRUE);
   
  /* Add all items to toolbar */
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tb_save, -1);
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tb_zoom_in, -1);
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tb_zoom_out, -1);
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tb_close, -1);

  g_signal_connect(G_OBJECT(tb_save), "clicked",
		   G_CALLBACK(callback_save_report), canvas);
  g_signal_connect(G_OBJECT(tb_zoom_in), "clicked",
		   G_CALLBACK(callback_report_zoom_in), canvas);
  g_signal_connect(G_OBJECT(tb_zoom_out), "clicked",
		   G_CALLBACK(callback_report_zoom_out), canvas);
  g_signal_connect_swapped(G_OBJECT(tb_close), "clicked",
			   G_CALLBACK(gtk_widget_destroy), G_OBJECT(window));

  /* Add toolbar HildonWindow */
#if PLAINGTK == 1
  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(toolbar), FALSE, FALSE, 5);
#else
  hildon_window_add_toolbar(window, GTK_TOOLBAR(toolbar));
#endif
}

/* gchar *report_conv_cost_label(struct lconv *locdata, const char *s) */
gchar *report_conv_cost_label(const char *s)
{
  if (strncmp(PRICEPERUNITTAG,s,strlen(PRICEPERUNITTAG))==0) {
    if (curunit.volumeunit==SI)
      return g_strdup(_("Avg. price/litre"));
  else
      return g_strdup(_("Avg. price/gallon"));
  }
  if (strncmp(CONSUMETAG,s,strlen(CONSUMETAG))==0) {
    if (curunit.consumeunit==SI)
      return g_strdup(_("Avg. l/100 km"));
  else
      return g_strdup(_("Avg. miles/gallon"));
  }
  if (strncmp(COSTTAG,s,strlen(COSTTAG))==0)
    return g_strjoin(" ",_("Cost"), curunit.currency->str, NULL);
  if (strncmp(COSTPERKMTAG,s,strlen(COSTPERKMTAG))==0)
    return g_strconcat(_("Cost"), " ", curunit.currency->str, "/",
		       (curunit.lengthunit==SI)? _("km") : _("mile"), 
		       NULL);
  else
    return g_strdup(s);
}

void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
{
/* #if PLAINGTK == 1 */
/*   GtkWidget *reportwindow; */
/* #else */
/*   HildonWindow *reportwindow; */
/* #endif */
  GtkWidget *vbox1;
  GtkWidget *button;
  GtkPlotCanvasChild *child;
  GdkColor color;
  gint page_width, page_height;
  gfloat scale;
  gdouble texty;
  gdouble textx;

  gchar *autotekstit[] = {gettext_noop("Car"),
			  gettext_noop("Car"),
			  gettext_noop("Model year"),
			  gettext_noop("Register"});
  gchar *kulutekstit[] = {gettext_noop("Overall expenses"),
			  gettext_noop("Fuel"), 
			  gettext_noop("Oil"),
			  gettext_noop("Service"), 
			  gettext_noop("Tires"),
			  gettext_noop("Total cost")};
  gchar *kmtekstit[] = {gettext_noop("Distance"), 
			gettext_noop("Beginning of year"), 
			gettext_noop("End of year"), 
			gettext_noop("Overall distance")};
  gchar *fueltekstit[] = {gettext_noop("Fuel"), 
			  gettext_noop("Total consumption"), 
			  COSTTAG, 
			  CONSUMETAG,
			  PRICEPERUNITTAG,
			  COSTPERKMTAG};
  gchar *oiltekstit[] = {gettext_noop("Oil"),
			 COSTTAG, 
			 COSTPERKMTAG};
  gchar *servicetekstit[] = {gettext_noop("Service"), 
			     COSTTAG, 
			     COSTPERKMTAG};
  gchar *tirestekstit[] = {gettext_noop("Tires"), 
			     COSTTAG, 
			     COSTPERKMTAG};
  gint i;

  gint lastyear;
  gchar *lastyearbuf;
  gchar car[30];
  gchar modelyear[30];
  gchar reg[30];
  gchar year[ENTRYDATEMAX];
  gchar startkm[ENTRYKMMAX];
  gchar endkm[ENTRYKMMAX];
  gchar yearkm[ENTRYKMMAX];
  gchar sumfill[ENTRYKMMAX]; 
  gchar sumprice[ENTRYKMMAX];
  gchar avgconsum[ENTRYKMMAX];
  gchar avgprice[ENTRYKMMAX];
  gchar pricekm[ENTRYKMMAX];
  gchar priceoil[ENTRYKMMAX];
  gchar priceoilkm[ENTRYKMMAX];
  gchar priceservice[ENTRYKMMAX];
  gchar priceservicekm[ENTRYKMMAX];
  gchar pricetires[ENTRYKMMAX];
  gchar pricetireskm[ENTRYKMMAX];
  gchar totalprice[ENTRYDATEMAX];

 if (arg1 == GTK_RESPONSE_ACCEPT) {

   lastyearbuf = gtk_combo_box_get_active_text(GTK_COMBO_BOX(pui->yearcombo));
   lastyear=atoi(lastyearbuf);
   g_free(lastyearbuf);

   if (gtk_combo_box_get_active(GTK_COMBO_BOX(pui->papercombo))==0) {
     page_width = GTK_PLOT_A4_W;
     page_height = GTK_PLOT_A4_H;
     scale = 670.0/GTK_PLOT_A4_W;
   }
   else {
     page_width = GTK_PLOT_LETTER_W;
     page_height = GTK_PLOT_LETTER_H;
     scale = 670.0/GTK_PLOT_LETTER_W;
   }

   /* Destroy the report settings dialog */
   gtk_widget_destroy(GTK_WIDGET(dialog));

   if (carchanged)
     update_car_changed(pui);

#if PLAINGTK == 1
   pui->app->reportwindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(pui->app->reportwindow), "Report");
   gtk_widget_set_usize(pui->app->reportwindow,800,600);
   gtk_container_border_width(GTK_CONTAINER(pui->app->reportwindow),0);
#else
   pui->app->reportwindow = HILDON_WINDOW(hildon_window_new());
   hildon_program_add_window(pui->app->program, pui->app->reportwindow);

  /* Callback for hardware keys */
  g_signal_connect(G_OBJECT(pui->app->reportwindow), "key_press_event",
		   G_CALLBACK(report_key_press_cb), pui);
#endif

  g_signal_connect (G_OBJECT(pui->app->reportwindow), "destroy",
		    G_CALLBACK (destroy_reportwin), pui);

   vbox1=gtk_vbox_new(FALSE,0);
   gtk_container_add(GTK_CONTAINER(pui->app->reportwindow),vbox1);
   gtk_widget_show(vbox1);

   pui->reportcanvas = gtk_plot_canvas_new(page_width, page_height, scale);

#if PLAINGTK == 1
   create_reportwin_menu(pui->app->reportwindow, vbox1, pui);
#else
   create_reportwin_menu(pui->app->reportwindow, pui);
#endif


#if PLAINGTK == 1
   create_report_toolbar(pui->app->reportwindow, vbox1, pui->reportcanvas);
#else
   create_report_toolbar(pui->app->reportwindow, pui->reportcanvas);
#endif

   pui->reportscrollwin=gtk_scrolled_window_new(NULL, NULL);
   gtk_container_border_width(GTK_CONTAINER(pui->reportscrollwin),0);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pui->reportscrollwin),
				  GTK_POLICY_ALWAYS,GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(vbox1),pui->reportscrollwin, TRUE, TRUE,0);
   gtk_widget_show(pui->reportscrollwin);

   /*  GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(pui->reportcanvas), GTK_PLOT_CANVAS_CAN_SELECT_ITEM); */
   /*  GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(pui->reportcanvas), GTK_PLOT_CANVAS_DND_FLAGS); */
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pui->reportscrollwin), pui->reportcanvas);
   /*  gtk_box_pack_start(GTK_BOX(vbox1), canvas, TRUE, TRUE, 0); */

   /*  gdk_color_parse("light blue", &color); */
   /*  gdk_color_alloc(gtk_widget_get_colormap(pui->reportcanvas), &color); */
   /*  gtk_plot_canvas_set_background(GTK_PLOT_CANVAS(pui->reportcanvas), &color); */

   gtk_widget_show(pui->reportcanvas);

   texty=report_init_line(HEADERLINE);
   report_put_text(pui->reportcanvas, .5, texty, "Helvetica-Bold", HEADING1SIZE, 
		   GTK_JUSTIFY_CENTER,
		   _("Car expenses"));
   texty=report_line_down(texty, HEADING1SIZE, 3.0, page_height);

   report_put_text(pui->reportcanvas, LEFTMARGIN, texty, "Helvetica-Bold", 
		   HEADING2SIZE, GTK_JUSTIFY_LEFT, gettext(autotekstit[0]));
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

   if (SQLITE_OK == sqlite3_bind_int(ppStmtCurCar, 1, currentcar)) {
     if (SQLITE_ROW == sqlite3_step(ppStmtCurCar)) {
       g_snprintf(car, 30, "%s %s", 
		  sqlite3_column_text(ppStmtCurCar, 0),
		  sqlite3_column_text(ppStmtCurCar, 1));
       g_snprintf(reg, 30, "%s", sqlite3_column_text(ppStmtCurCar, 2));
       g_snprintf(modelyear, 30, "%s", sqlite3_column_text(ppStmtCurCar, 3));
     }
     sqlite3_reset(ppStmtCurCar);
   }
   for (i=1;i<sizeof(autotekstit)/sizeof(char *);i++) {
     report_put_text(pui->reportcanvas, LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, 
		     GTK_JUSTIFY_LEFT,
		     gettext(autotekstit[i]));
     texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   }

   texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
   report_put_text(pui->reportcanvas,
		   LEFTMARGIN, texty, "Helvetica-Bold", HEADING2SIZE, GTK_JUSTIFY_LEFT,
		   gettext(kmtekstit[0]));
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

   for (i=1;i<sizeof(kmtekstit)/sizeof(char *);i++) {
     char *str;
     str=report_conv_cost_label(gettext(kmtekstit[i]));
     report_put_text(pui->reportcanvas, LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, GTK_JUSTIFY_LEFT,
		     str);
     g_free(str);
     texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   }

   texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
   report_put_text(pui->reportcanvas,
		   LEFTMARGIN, texty, "Helvetica-Bold", HEADING2SIZE, GTK_JUSTIFY_LEFT,
		   gettext(fueltekstit[0]));
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

   for (i=1;i<sizeof(fueltekstit)/sizeof(char *);i++) {
     char *str;
     str=report_conv_cost_label(gettext(fueltekstit[i]));
     report_put_text(pui->reportcanvas,
		     LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, GTK_JUSTIFY_LEFT,
		     str);
     g_free(str);
     texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   }

   texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
   report_put_text(pui->reportcanvas,
		   LEFTMARGIN, texty, "Helvetica-Bold", HEADING2SIZE, GTK_JUSTIFY_LEFT,
		   gettext(oiltekstit[0]));
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

   for (i=1;i<sizeof(oiltekstit)/sizeof(char *);i++) {
     char *str;
     str=report_conv_cost_label(gettext(oiltekstit[i]));
     g_print("%s\n",str);
     report_put_text(pui->reportcanvas,
		     LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, GTK_JUSTIFY_LEFT,
		     str);
     g_free(str);
     texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   }

   texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
   report_put_text(pui->reportcanvas,
		   LEFTMARGIN, texty, "Helvetica-Bold", HEADING2SIZE, GTK_JUSTIFY_LEFT,
		   gettext(servicetekstit[0]));
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

   for (i=1;i<sizeof(servicetekstit)/sizeof(char *);i++) {
     char *str;
     str=report_conv_cost_label(gettext(servicetekstit[i]));
     report_put_text(pui->reportcanvas,
		     LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, GTK_JUSTIFY_LEFT,
		     str);
     g_free(str);
     texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   }

   texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
   report_put_text(pui->reportcanvas,
		   LEFTMARGIN, texty, "Helvetica-Bold", HEADING2SIZE, GTK_JUSTIFY_LEFT,
		   gettext(tirestekstit[0]));
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

   for (i=1;i<sizeof(tirestekstit)/sizeof(char *);i++) {
     char *str;
     str=report_conv_cost_label(gettext(tirestekstit[i]));
     report_put_text(pui->reportcanvas,
		     LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, GTK_JUSTIFY_LEFT,
		     str);
     g_free(str);
     texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   }

   texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
   report_put_text(pui->reportcanvas,
		   LEFTMARGIN, texty, "Helvetica-Bold", HEADING2SIZE, GTK_JUSTIFY_LEFT,
		   gettext(kulutekstit[0]));
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

   for (i=1;i<sizeof(kulutekstit)/sizeof(char *);i++) {
     char *str;
     str=report_conv_cost_label(gettext(kulutekstit[i]));
     report_put_text(pui->reportcanvas,
		     LEFTMARGIN+INDENT, texty, "Helvetica", NORMALSIZE, GTK_JUSTIFY_LEFT,
		     str);
     g_free(str);
     texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   }

   texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);

   texty=report_init_line(HEADERLINE);
   texty=report_line_down(texty, HEADING1SIZE, 3.0, page_height);
   texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);
   report_put_text(pui->reportcanvas, FIRSTCOLUMN, texty, "Helvetica", NORMALSIZE,
		   GTK_JUSTIFY_LEFT,
		   car);
   texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   report_put_text(pui->reportcanvas, FIRSTCOLUMN, texty, "Helvetica", NORMALSIZE,
		   GTK_JUSTIFY_LEFT,
		   modelyear);
   texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
   report_put_text(pui->reportcanvas, FIRSTCOLUMN, texty, "Helvetica", NORMALSIZE,
		   GTK_JUSTIFY_LEFT,
		   reg);
   texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);

   /* Fill the column with data */
   if (SQLITE_OK == sqlite3_bind_int(ppStmtGetReport, 1, currentcar)) {
/*      sqlite3_bind_int(ppStmtGetReport, 2, 2001); */
/*      sqlite3_bind_int(ppStmtGetReport, 3, 2002); */
     textx = FIRSTCOLUMN;
     while(SQLITE_ROW == sqlite3_step(ppStmtGetReport)) {
       g_snprintf(year, ENTRYDATEMAX, "%s",
		  sqlite3_column_text(ppStmtGetReport,0));
       if (atoi(year) > lastyear) break;
       if (atoi(year) > lastyear-REPORTYEARS) {
	 g_snprintf(startkm, ENTRYKMMAX, "%.0f",
		    SIlength2user(sqlite3_column_double(ppStmtGetReport,1)));
	 g_snprintf(endkm, ENTRYKMMAX, "%.0f",
		    SIlength2user(sqlite3_column_double(ppStmtGetReport,2)));
	 g_snprintf(yearkm, ENTRYKMMAX, "%.0f",
		    SIlength2user(sqlite3_column_double(ppStmtGetReport,3)));
	 g_snprintf(sumfill, ENTRYKMMAX, "%.1f",
		    SIvolume2user(sqlite3_column_double(ppStmtGetReport,4)));
	 g_snprintf(sumprice, ENTRYKMMAX, "%.1f",
		    sqlite3_column_double(ppStmtGetReport,5));
	 g_snprintf(avgconsum, ENTRYKMMAX, "%.1f",
		    SIconsumption2user(sqlite3_column_double(ppStmtGetReport,6)));
	 g_snprintf(avgprice, ENTRYKMMAX, "%.2f",
		    SIppl2user(sqlite3_column_double(ppStmtGetReport,7)));
	 g_snprintf(pricekm, ENTRYKMMAX, "%.2f",
		    user2SIlength(sqlite3_column_double(ppStmtGetReport,8)));
	 g_snprintf(priceoil, ENTRYKMMAX, "%.2f",
		    sqlite3_column_double(ppStmtGetReport,9));
	 g_snprintf(priceoilkm, ENTRYKMMAX, "%.2f",
		    user2SIlength(sqlite3_column_double(ppStmtGetReport,10)));
	 g_snprintf(priceservice, ENTRYKMMAX, "%.2f",
		    sqlite3_column_double(ppStmtGetReport,11));
	 g_snprintf(priceservicekm, ENTRYKMMAX, "%.2f",
		    user2SIlength(sqlite3_column_double(ppStmtGetReport,12)));
	 g_snprintf(pricetires, ENTRYKMMAX, "%.2f",
		    sqlite3_column_double(ppStmtGetReport,13));
	 g_snprintf(pricetireskm, ENTRYKMMAX, "%.2f",
		    user2SIlength(sqlite3_column_double(ppStmtGetReport,14)));

	 /* Year header*/
	 texty=report_init_line(HEADERLINE);
	 texty=report_line_down(texty, HEADING1SIZE, 3.0, page_height);
	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);
	 texty=report_line_down(texty, NORMALSIZE, ((sizeof(autotekstit)/sizeof(char *))-1)*1.2, page_height);
	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, year);
	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

	 /* Km */
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, startkm);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, endkm);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, yearkm);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);

	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

	 /* Fuel */
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, sumfill);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, sumprice);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, avgconsum);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, avgprice);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, pricekm);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);

	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

	 /* Oil */
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, priceoil);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, priceoilkm);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);

	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

	 /* Service */
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, priceservice);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, priceservicekm);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);

	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

	 /* Tires */
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, pricetires);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, pricetireskm);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);

	 texty=report_line_down(texty, HEADING2SIZE, 1.0, page_height);
	 texty=report_line_down(texty, HEADING2SIZE, 2.0, page_height);

	 /* Overall expenses */
	 g_snprintf(totalprice, ENTRYDATEMAX, "%.2f", 
		    atof(sumprice)+atof(priceoil)+atof(priceservice)
		    +atof(pricetires));

	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, sumprice);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, priceoil);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, priceservice);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, pricetires);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
			 GTK_JUSTIFY_LEFT, totalprice);
	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);


	 /* Next column*/
	 textx += COLUMNSEP;
       }
     }
     sqlite3_reset(ppStmtGetReport);
   }
   else
     g_print("Something wrong in obtaining the report data\n");


   gtk_widget_show_all(GTK_WIDGET(pui->app->reportwindow));
   if (pui->mainfullscreen)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->report_menu_item_fullscreen),
				    !pui->reportfullscreen);
 }
 else {
   /* Destroy the report settings dialog */
   gtk_widget_destroy(GTK_WIDGET(dialog));
 }
}

void callback_report( GtkAction * action, AppUIData *pui )
{
  GtkWidget *dialog;
  GtkWidget *vbox;
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *button;

  if (pui->app->reportwindow != NULL) {
    gtk_window_present(GTK_WINDOW(pui->app->reportwindow));
    return;
  }

  if (!db_open()) {
    dialog = gtk_dialog_new_with_buttons(_("Report"),
					 GTK_WINDOW(pui->app->mainwindow),
					 GTK_DIALOG_MODAL,
					 GTK_STOCK_OK,
					 GTK_RESPONSE_REJECT,
					 NULL);
    vbox = GTK_WIDGET(GTK_DIALOG(dialog)->vbox);

    label=gtk_label_new(_("Can't access database - reporting not possible"));
    gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 5);
    gtk_widget_show(label);
  }
  else {
    dialog = gtk_dialog_new_with_buttons(_("Report"),
					 GTK_WINDOW(pui->app->mainwindow),
					 GTK_DIALOG_MODAL,
					 _("Generate"), GTK_RESPONSE_ACCEPT,
					 GTK_STOCK_CANCEL,
					 GTK_RESPONSE_REJECT,
					 NULL);
    vbox = GTK_WIDGET(GTK_DIALOG(dialog)->vbox);

    table = gtk_table_new(3, 3, FALSE);
    gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 5);

    label=gtk_label_new(_("Car:"));
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
		     GTK_EXPAND|GTK_FILL,
		     0, 0, 5);
    gtk_widget_show(label);

    pui->carcombo = create_car_combo();
    gtk_table_attach(GTK_TABLE(table), pui->carcombo, 1, 2, 0, 1, 
		     GTK_EXPAND|GTK_FILL,
		     0, 0, 5);
    gtk_widget_show(pui->carcombo);

    label=gtk_label_new(_("Last year:"));
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 
		     GTK_EXPAND|GTK_FILL,
		     0, 0, 5);
    gtk_widget_show(label);

    pui->yearcombo = create_year_combo();
    gtk_table_attach(GTK_TABLE(table), pui->yearcombo, 1, 2, 1, 2,
		     GTK_EXPAND|GTK_FILL,
		     0, 0, 5);
    gtk_widget_show(pui->yearcombo);

    button = gtk_button_new_with_label(_("Update"));
    gtk_table_attach(GTK_TABLE(table), button, 2, 3, 1, 2,
		     GTK_EXPAND|GTK_FILL,
		     0, 0, 5);
    gtk_widget_show(button);
    g_signal_connect (GTK_OBJECT (button), "clicked",
			G_CALLBACK(update_yearcombo_cb), pui->yearcombo);

    label=gtk_label_new(_("Paper:"));
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
		     GTK_EXPAND|GTK_FILL,
		     0, 0, 5);
    gtk_widget_show(label);

    pui->papercombo = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (pui->papercombo),"A4");
    gtk_combo_box_append_text(GTK_COMBO_BOX (pui->papercombo),"Letter");
    gtk_combo_box_set_active(GTK_COMBO_BOX (pui->papercombo),0);
    gtk_table_attach(GTK_TABLE(table), pui->papercombo, 1, 2, 2, 3,
		     GTK_EXPAND|GTK_FILL,
		     0, 0, 5);
    gtk_widget_show(pui->papercombo);

    gtk_widget_show(table);

  }

  hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
				 HELP_ID_REPORT,
				 pui->app->osso);

/*   g_signal_connect_swapped (dialog, "response", */
/* 			    G_CALLBACK (report_response), */
/* 			    dialog); */
  g_signal_connect (dialog, "response",
		    G_CALLBACK (report_response),
		    pui);
  gtk_widget_show(dialog);

}
