/* 
 * This file is part of fuelpad, the fuel diary
 *
 * Copyright (c) 2007, 2008 Julius Luukko <julle.luukko@quicknet.inet.fi>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ui.h"
#include "uistat.h"

/* Statistics window related*/
#define X1NOFULL_FP .15
#define Y1NOFULL_FP .1
#define X2NOFULL_FP .85
#define Y2NOFULL_FP .85
#define XNOFULL_NEXT .95
#define YNOFULL_NEXT .95
#define XNOFULL_PREV .05
#define YNOFULL_PREV .95

int currentyear;

extern unit_t curunit;

/**
 * \fn gboolean delete_statwin( GtkWidget *widget, GdkEvent  *event, gpointer   data )
 * \brief Statistics window delete_event callback
 * \return FALSE
 */
static gboolean delete_statwin( GtkWidget *widget,
                                GdkEvent  *event,
                                gpointer   data )
{
  extern AppUIData ui;

  gtk_widget_destroy(GTK_WIDGET(ui.app->statwindow));
  return FALSE;
}

void destroy_statwin( GtkWidget *widget,
		      gpointer   data )
{
  AppUIData *a;

  a=(AppUIData *)data;

  gtk_widget_destroy(GTK_WIDGET(a->app->statwindow));
  a->app->statwindow=NULL;
}

void
plot_consumption(AppUIData *pui)
{
  static GtkPlotData *dataset;

  char yearbuf[YEARBUFLEN];
  GdkColor color;
  static double px2[12] = {0.0};
  static double py2[12] = {0.0};
  double ymax = 0.0;
  int nummonths;
  int i;

  gdk_color_parse("blue", &color);
  gdk_color_alloc(gdk_colormap_get_system(), &color); 

  if (dataset!=NULL)
    gtk_plot_remove_data(GTK_PLOT(pui->active_plot), dataset);
  dataset = GTK_PLOT_DATA(gtk_plot_bar_new(GTK_ORIENTATION_VERTICAL));
  gtk_plot_add_data(GTK_PLOT(pui->active_plot), dataset);
  gtk_widget_show(GTK_WIDGET(dataset));

  for (i=0;i<12;i++) {
    py2[i]=0.0;
    px2[i]=i+1;
  }

  if (currentyear==0)
    currentyear=getyear();
  nummonths=db_get_monthly_data(currentyear,px2,py2);
  for (i=0;i<12;i++) {
    if (curunit.consumeunit != SI)
      if (py2[i] > 0.0001)
	py2[i]=SIconsumption2user(py2[i]);
    if (py2[i] > ymax) ymax=py2[i];
  }
  gtk_plot_data_set_points(dataset, px2, py2, NULL, NULL, nummonths);
  if (ymax>0.0001)
    gtk_plot_set_yrange(GTK_PLOT(pui->active_plot), 0., 1.1*ymax);
  else
    if (curunit.consumeunit==SI)
      gtk_plot_set_yrange(GTK_PLOT(pui->active_plot), 0., 13.0);
    else
      gtk_plot_set_yrange(GTK_PLOT(pui->active_plot), 0., 30.0);

  gtk_plot_data_set_symbol(dataset,
			   GTK_PLOT_SYMBOL_NONE,
			   GTK_PLOT_SYMBOL_OPAQUE,
			   10, 2, &color, &color);

  gtk_plot_data_set_line_attributes(dataset,
				    GTK_PLOT_LINE_NONE,
				    0, 0, 1, &color);
  g_snprintf(yearbuf,YEARBUFLEN,"%s %d",_("Year"),currentyear);
  gtk_plot_data_set_legend(dataset, yearbuf);

  gtk_plot_bar_set_width(GTK_PLOT_BAR(dataset), 0.25);

  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(pui->canvas));
  gtk_widget_queue_draw(GTK_WIDGET(pui->canvas));

  gtk_plot_axis_set_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), pui->months);
  gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), TRUE);
}

void browse_prev_year(AppUIData *pui)
{
  currentyear-=1;
  plot_consumption(pui);
}

void browse_next_year(AppUIData *pui)
{
  currentyear+=1;
  plot_consumption(pui);
}

void browse_this_year(AppUIData *pui)
{
  currentyear=getyear();
  plot_consumption(pui);
}

void stat_window_fullscreen(gpointer data)
{
  AppUIData *a;

  a=(AppUIData *)data;
  a->statfullscreen = !a->statfullscreen;
  if (a->statfullscreen) {
    gtk_plot_canvas_set_magnification(GTK_PLOT_CANVAS(a->canvas), 800.0/GTK_PLOT_A4_W);
    gtk_plot_canvas_paint(GTK_PLOT_CANVAS(a->canvas));
    gtk_window_fullscreen(GTK_WINDOW(a->app->statwindow));
  } else {
    gtk_plot_canvas_set_magnification(GTK_PLOT_CANVAS(a->canvas), 720.0/GTK_PLOT_A4_W);
    gtk_plot_canvas_paint(GTK_PLOT_CANVAS(a->canvas));
    gtk_window_unfullscreen(GTK_WINDOW(a->app->statwindow));
  }
}

void cb_stat_fullscreen( GtkAction * action, gpointer data )
{
  stat_window_fullscreen(data);
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(((AppUIData *)(data))->stat_menu_item_fullscreen),
				 ((AppUIData *)(data))->statfullscreen);
}

#if PLAINGTK == 0
gboolean stat_key_press_cb(GtkWidget * widget, GdkEventKey * event,
			   AppUIData *pui)
{
  switch (event->keyval) {

    case GDK_F6:
      gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->stat_menu_item_fullscreen),
				     !pui->statfullscreen);
      return TRUE;
    case GDK_Left:
      browse_prev_year(pui);
      return TRUE;
      break;
    case GDK_Right:
      browse_next_year(pui);
      return TRUE;
      break;
    case GDK_Escape:
      browse_this_year(pui);
      return TRUE;
      break;
    }

    return FALSE;
}
#endif

gint
select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *child, 
            AppUIData *pui)
{
  if(GTK_IS_PLOT_CANVAS_TEXT(child)) {
	if(strncmp(GTK_PLOT_CANVAS_TEXT(child)->text.text,_("Next"),
		   strlen(_("Next")))==0)
	  browse_next_year(pui);
	if(strncmp(GTK_PLOT_CANVAS_TEXT(child)->text.text,_("Prev"),
		   strlen(_("Prev")))==0)
	  browse_prev_year(pui);
  }
  return TRUE;
}

GtkWidget *
new_layer(AppUIData *pui)
{
  static gint nlayers = 0;

  nlayers++;

  pui->plots = (GtkWidget **)g_realloc(pui->plots, nlayers * sizeof(GtkWidget *));

  pui->plots[nlayers-1] = gtk_plot_new_with_size(NULL, .5, YSCALE(.25));
  gtk_widget_show(pui->plots[nlayers-1]);

  return pui->plots[nlayers-1];
}

#if PLAINGTK == 1
void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
#else
  void create_statwin_menu(HildonWindow *window, AppUIData *pui)
#endif
{
  GtkWidget *main_menu;
  GtkWidget *menu_view;
  GtkWidget *item_view;
  GtkWidget *item_exit;
#if PLAINGTK == 1
  GtkWidget *menubar;
  GtkWidget *item_file;
#endif
 
  main_menu = gtk_menu_new();
  menu_view = gtk_menu_new();

  /* Create main menu items */
  item_view = gtk_menu_item_new_with_label(_("View"));
  item_exit = gtk_menu_item_new_with_label(_("Close window"));

  /* Create view menu items */
  pui->stat_menu_item_fullscreen = gtk_check_menu_item_new_with_label(_("Full screen"));
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->stat_menu_item_fullscreen),
				 pui->statfullscreen);

  /* Add menu items to right menus */
  /* Main menu */
  gtk_menu_append(main_menu, item_view);
  gtk_menu_append(main_menu, gtk_separator_menu_item_new());
  gtk_menu_append(main_menu, item_exit);

  /* View menu */
  gtk_menu_append(menu_view, pui->stat_menu_item_fullscreen);

#if PLAINGTK == 1
  menubar = gtk_menu_bar_new();
  gtk_box_pack_start (GTK_BOX (root), menubar, FALSE, FALSE, 2);
  gtk_widget_show (menubar);
  item_file = gtk_menu_item_new_with_label ("fuelpad");
  gtk_widget_show (item_file);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (item_file), main_menu);
  gtk_menu_bar_append (GTK_MENU_BAR (menubar), item_file);
#else
  hildon_window_set_menu(HILDON_WINDOW(window), GTK_MENU(main_menu));
#endif

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_view), menu_view);

  /* Attach the callback functions to the activate signal */
  g_signal_connect_swapped(G_OBJECT(item_exit), "activate",
			   G_CALLBACK(delete_statwin), G_OBJECT(window));

  g_signal_connect(G_OBJECT(pui->stat_menu_item_fullscreen), "toggled",
		   G_CALLBACK(cb_stat_fullscreen), pui);

  /* Make all menu widgets visible */
  gtk_widget_show_all(GTK_WIDGET(main_menu));
}

void callback_stat( GtkAction * action, AppUIData *pui )
{
/* #if PLAINGTK == 1 */
/*   GtkWidget *statwindow; */
/* #else */
/*   HildonWindow *statwindow; */
/* #endif */
  GtkPlotCanvasChild *fuelplot;
  GtkPlotCanvasChild *prevtext;
  GtkPlotCanvasChild *nexttext;
  GtkWidget *vbox1;
  GtkPlotCanvasChild *child;
  GdkColor color;
  gint page_width, page_height;
  gfloat scale = 720.0/GTK_PLOT_A4_W;

  static gchar *month_labels[] = {"\0","Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

  if (pui->app->statwindow != NULL) {
    gtk_window_present(GTK_WINDOW(pui->app->statwindow));
    return;
  }

  page_width = GTK_PLOT_A4_W;
  page_height = GTK_PLOT_A4_H;

#if PLAINGTK == 1
  pui->app->statwindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(pui->app->statwindow), "Plot");
  gtk_widget_set_usize(pui->app->statwindow,800,400);
  gtk_container_border_width(GTK_CONTAINER(pui->app->statwindow),0);
#else
  pui->app->statwindow = HILDON_WINDOW(hildon_window_new());
  hildon_program_add_window(pui->app->program, pui->app->statwindow);

  /* Callback for hardware keys */
  g_signal_connect(G_OBJECT(pui->app->statwindow), "key_press_event", 
		   G_CALLBACK(stat_key_press_cb), pui);
#endif

  g_signal_connect(G_OBJECT(pui->app->statwindow), "destroy",
		   G_CALLBACK(destroy_statwin), pui);

  vbox1=gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(pui->app->statwindow),vbox1);
  gtk_widget_show(vbox1);

#if PLAINGTK == 1
  create_statwin_menu(pui->app->statwindow, vbox1, pui);
#else
  create_statwin_menu(pui->app->statwindow, pui);
#endif

  pui->canvas = gtk_plot_canvas_new(page_width, page_height, scale);
  gtk_widget_set_usize(pui->canvas,page_width,page_height);
  GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(pui->canvas), GTK_PLOT_CANVAS_CAN_SELECT_ITEM);
  gtk_box_pack_start(GTK_BOX(vbox1), pui->canvas, TRUE, TRUE, 0);

  gdk_color_parse("light blue", &color);
  gdk_color_alloc(gtk_widget_get_colormap(pui->canvas), &color);
  gtk_plot_canvas_set_background(GTK_PLOT_CANVAS(pui->canvas), &color);

  gtk_widget_show(pui->canvas);

  pui->active_plot = new_layer(pui);
  gdk_color_parse("light yellow", &color);
  gdk_color_alloc(gtk_widget_get_colormap(pui->active_plot), &color);
  gtk_plot_set_background(GTK_PLOT(pui->active_plot), &color);

  gdk_color_parse("light blue", &color);
  gdk_color_alloc(gtk_widget_get_colormap(pui->canvas), &color);
  gtk_plot_legends_set_attributes(GTK_PLOT(pui->active_plot),
				  NULL, 0,
				  NULL,
				  &color);

  gtk_plot_set_xrange(GTK_PLOT(pui->active_plot), 0. , 13.0);

  gtk_plot_axis_set_labels_numbers(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_LEFT), GTK_PLOT_LABEL_FLOAT, 1);

  /* (*plot, vmajor, vminor, hmajor, hminor) */
  if (curunit.consumeunit==SI)
    gtk_plot_grids_set_visible(GTK_PLOT(pui->active_plot), TRUE, FALSE, TRUE, TRUE);
  else
    gtk_plot_grids_set_visible(GTK_PLOT(pui->active_plot), TRUE, FALSE, TRUE, FALSE);

  gtk_plot_axis_show_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_TOP), FALSE);
  gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_TOP), FALSE);
  gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_TOP));

  gtk_plot_axis_show_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_RIGHT), FALSE);
  gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_RIGHT), FALSE);
  gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_RIGHT));

  /* (GtkPlot *plot, GtkPlotBorderStyle border, gint shadow_width);*/
  gtk_plot_set_legends_border(GTK_PLOT(pui->active_plot), 2, 0);
  gtk_plot_legends_move(GTK_PLOT(pui->active_plot), .9, YSCALE(.05));
  /*  gtk_plot_hide_legends(GTK_PLOT(pui->active_plot)); */

  gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), _("Month"));
  /* It seems that the coordinates of bottom title do not work very
     well with scaling */
  gtk_plot_axis_move_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), 0, .4, YSCALE(0.6));
  gtk_plot_axis_justify_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), GTK_JUSTIFY_CENTER);
  if (curunit.consumeunit==SI)
    gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_LEFT), _("Fuel consumption [l/100 km]"));
  else
    gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_LEFT), _("Miles per gallon"));

  gtk_widget_show(pui->active_plot);

  fuelplot = gtk_plot_canvas_plot_new(GTK_PLOT(pui->active_plot));
  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(pui->canvas), fuelplot, X1NOFULL_FP, YSCALE(Y1NOFULL_FP), X2NOFULL_FP, YSCALE(Y2NOFULL_FP));
  gtk_widget_show(pui->active_plot);

  g_signal_connect(GTK_OBJECT(pui->canvas), "select_item",
		   G_CALLBACK(select_item), pui);

  child = gtk_plot_canvas_text_new("Helvetica-Bold", 16, 0, NULL, NULL, TRUE,
				   GTK_JUSTIFY_CENTER,
				   _("Average monthly fuel consumption"));
  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(pui->canvas), child, .50, YSCALE(.050), .0, .0);

  nexttext = gtk_plot_canvas_text_new("Helvetica", 16, 0, NULL, NULL, TRUE,
				   GTK_JUSTIFY_RIGHT,
				   _("Next"));
  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(pui->canvas), nexttext, XNOFULL_NEXT, YSCALE(YNOFULL_NEXT), .0, .0);

  prevtext = gtk_plot_canvas_text_new("Helvetica", 16, 0, NULL, NULL, TRUE,
				   GTK_JUSTIFY_LEFT,
				   _("Prev"));
  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(pui->canvas), prevtext, XNOFULL_PREV, YSCALE(YNOFULL_PREV), .0, .0);

  plot_consumption(pui);

  if (curunit.consumeunit==SI)
    gtk_plot_axis_set_major_ticks(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_LEFT), 1.0);
  else
    gtk_plot_axis_set_major_ticks(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_LEFT), 2.0);
  gtk_plot_axis_set_major_ticks(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), 1.0);
  pui->months = GTK_PLOT_ARRAY(gtk_plot_array_new(NULL, month_labels, 13, GTK_TYPE_STRING, FALSE));
  gtk_plot_axis_set_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), pui->months);
  gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), TRUE);

  gtk_widget_show_all(GTK_WIDGET(pui->app->statwindow));
  if (pui->mainfullscreen) 
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->stat_menu_item_fullscreen),
				   !pui->statfullscreen);
}
